#!/usr/bin/env python3
#prog_06_03.py

import tkinter
import uno_slave
import plotter
import time
import threading

class TransistorPlot(tkinter.Frame, threading.Thread):
    def __init__(self, master):
        self.master = master
        self.uno1 = uno_slave.Uno(address = 1, project = 3, config_digital = [])
        self.master.title('2N222A Common Emitter Output Charcteristics ')
        self.Y = 400
        self.X = 400
        self.max_collector_voltage = 8
        self.max_collector_current = 70e-3    
        self.graph1 = plotter.Graph(master,self.X, self.Y, scale_x = self.max_collector_voltage, scale_y = self.max_collector_current)    
        self.btn_test = tkinter.Button(master,text = 'Test', command = self.start_test)
        self.lbl_base = tkinter.Label(justify = 'left')
        self.lbl_collector = tkinter.Label(justify = 'left')
        
        self.graph1.frm.grid(row = 0, column = 0, padx = 10, pady =10, columnspan = 3)   
        self.lbl_base.grid(row = 1, column = 0, sticky = 'NW', padx = 5, pady = 5)
        self.lbl_collector.grid(row = 1, column = 1, sticky = 'NW', padx = 5, pady = 5)
        self.btn_test.grid(row = 1, column = 2, pady = 10)
        
    def start_test(self):
        self.btn_test.configure(state = 'disabled')
        tr = threading.Thread(target=self.test, args = ())
        tr.setDaemon(True)
        tr.start()
            
    def test(self):
        self.measurements = []
        for a in range (0,6):
            self.measurements.append([])
            for x in range (0, self.X):
                self.measurements[a].append(0)
        self.i_base = 0
        self.i_collector = 0
        self.v_input = 0
        self.v_supply = 0
        self.v_collector = 0
        self.v_base = 0 
        self.uno1.write_analog_dict = {5:0, 6:0} 
        self.uno1.write_analog()
        time.sleep(1)
        base_currents = [30e-6, 60e-6, 90e-6, 120e-6, 150e-6, 180e-6]  
        for channel, ib in enumerate(base_currents): 
            self.uno1.write_analog_dict[6] = 0
            self.uno1.write_analog()            
            self.measure_collector()
            for x in range (0, self.X):
                self.set_Ib(ib)       
                vc = x*self.max_collector_voltage/self.X
                self.set_Vce(vc)
                measurement = []
                for n in range (0,2):
                    self.measure_collector()
                    measurement.append(self.i_collector)
                
                self.measurements[channel][x] = sum(measurement)/len(measurement)
                self.measure_base()
                
            
            self.graph1.reset()            
            self.plot_measurements()

        self.uno1.write_analog_dict = {5:0, 6:0}
        self.uno1.write_analog()
        self.btn_test.configure(state = 'normal')      

    def measure_base(self):
        self.uno1.read_analog(2)
        self.v_input = self.uno1.read_analog_dict[0] * 5/1023
        self.v_base = self.uno1.read_analog_dict[1] * 5/1023
        self.i_base = (self.v_input - self.v_base)/22e3
        self.lbl_base.configure(text = 'i_base = {0:0.3f} µA\n\n'.format(self.i_base*1e6))     

    def measure_collector(self):
        self.uno1.read_analog(4)
        self.v_collector = self.uno1.read_analog_dict[3]*10/1023
        self.v_supply = self.uno1.read_analog_dict[2]*10/1023
        self.i_collector = (self.v_supply - self.v_collector)/22    
        self.lbl_collector.configure(text = 'i_collector = {1:0.3f} mA\n\
v_collector = {2:0.3f} V'.format(self.i_base * 1e6, self.i_collector * 1e3, self.v_collector))

    def set_Ib(self,ib):
        self.measure_base()
        if self.i_base > ib + 1e-6:
               self.uno1.write_analog_dict[5] -= 1

        while self.i_base < ib:
            self.uno1.write_analog_dict[5] += 1
            self.uno1.write_analog()
            time.sleep(1)
            self.measure_base()            
   
    def set_Vce(self, v):
        while self.v_collector < v:
            self.uno1.write_analog_dict[6] += 1    
            self.uno1.write_analog()
            time.sleep(1)
            self.measure_collector()

    def plot_measurements(self):
        for x in range (0,self.X):
            a = self.measurements[0][x]*self.Y/self.max_collector_current
            b = self.measurements[1][x]*self.Y/self.max_collector_current
            c = self.measurements[2][x]*self.Y/self.max_collector_current
            d = self.measurements[3][x]*self.Y/self.max_collector_current
            e = self.measurements[4][x]*self.Y/self.max_collector_current
            f = self.measurements[5][x]*self.Y/self.max_collector_current                
            self.graph1.plot(a, b, c, d, e, f)       

tk = tkinter.Tk()
front_end = TransistorPlot(tk)
tk.mainloop()