#!/usr/bin/env python3
#12F1622_PROGRAMMER_V1.0.py

from tkinter import *
import uno_slave
import threading
import time
from tkinter import filedialog as filedialog


DATA = 7 
CLK = 8  
NOT_MCLR = 9 

class Pic12f1822:
    def __init__(self,master):
        self.master = master
        
        self.master.title('PIC 12F1822 Current Meter Programmer')
        self.output_text = Text(width = 60, height = 30, relief = 'ridge') 
        self.btn_file = Button(text = 'Choose Hex File', command = self.choose_file)
        self.btn_load = Button(text = 'Load to PIC',command = self.start_load, state = DISABLED)
        self.btn_read = Button(text = 'Read from PIC', command = self.start_read, state = DISABLED)
        self.output_text.grid(row = 0, column = 0, columnspan = 3, pady = 5, padx = 5)  
        
        self.btn_file.grid(row = 1, column = 0, pady = 5)
        self.btn_load.grid(row = 1, column = 1, pady = 5)        
        self.btn_read.grid(row = 1, column = 2, pady = 5)
        self.p_write=[0]*0x8010
        self.p_read = {}
        '''See PIC data sheet  DS41390C'''
        self.enter_program_verify_i = [0, 0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 1, 0, 0, 1, 0, 1, 1, 0, 0, 0, 0, 1, 0, 1, 0, 1, 1, 0, 0, 1, 0]                             
        self.load_program_memory_i = self.make_list(0x02, 6)        
        self.load_data_memory_i = self.make_list(0x03,6)
        self.read_program_memory_i = self.make_list(0x04,6)
        self.read_data_memory_i = self.make_list(0x05,6)        
        self.increment_address_i = self.make_list(0x06, 6)
        self.begin_programming_internally_timed_i = self.make_list(0x08,6)
        self.bulk_erase_program_memory_i = self.make_list(0x09, 6)
        self.reset_address_i = self.make_list(0x16, 6)

    def choose_file(self):
        file_name = filedialog.askopenfilename(filetypes = [('Hex','*.hex')],title = 'Choose Hex File')
        self.btn_load.config(state = NORMAL)
        self.btn_read.config(state = NORMAL)
        self.btn_file.config(state = DISABLED)
        try:
            fhex = open(file_name)
            hex_lines = fhex.readlines()
            fhex.close()
            extended_linear_address = 0        
            for line in hex_lines:
                if line[0] == ':':
                    tempbuff = '0x'
                    tempbuff = tempbuff + line[1]+line[2]  
                    databytes = int(tempbuff,16)
                    tempbuff = '0x'
                    tempbuff = tempbuff + line[3] + line[4] + line[5] + line[6]
                    address = (extended_linear_address + int(tempbuff,16))//2
                    tempbuff = '0x'
                    tempbuff = tempbuff + line[7] + line[8]
                    recordtype=int(tempbuff,16)
                    words=databytes//2
                    if recordtype == 0: 
                      m=0               
                      for m in range (0,words): 
                        tempbuff = '0x' + line[11+m*4] + line[12+m*4] + line[9+m*4] + line[10+m*4]
                        self.p_write[address+m] = int(tempbuff,16)
                        self.output_text.insert(END, '0x{0:x} 0x{1:x}\n'.format(address + m,self.p_write[address+m])) 
                        self.output_text.see(END)              
                        if (address + m) < 0x2001:
                                self.pc = address + m

                    if recordtype == 4:
                        tempbuff = '0x' + line[9]+ line[10]+ line[11]+ line[12]
                        extended_linear_address = int(tempbuff,16) << 16
                        self.output_text.insert(END,'extended linear address 0x{0:x}\n'.format(extended_linear_address)) 
                        self.output_text.see(END)     

        except Exception as e:
            self.output_text.insert(END, e) 
            self.btn_load.configure(state = DISABLED)  
            self.btn_read.configure(state = DISABLED)  

    def start_load(self):
        self.btn_read.configure(state = DISABLED)
        self.btn_load.configure(state = DISABLED)
        tr = threading.Thread(target=self.load, args = ())
        tr.setDaemon(True)
        tr.start()
             

    def start_read(self):
            self.btn_read.configure(state = DISABLED)
            self.btn_load.configure(state = DISABLED)
            tr = threading.Thread(target=self.read, args = ())
            tr.setDaemon(True)
            tr.start()
            
    def load(self):
        self.uno1 = uno_slave.Uno(address = 1, project = 4, config_digital = [DATA, CLK, NOT_MCLR])
        self.uno1.write_digital_dict = {DATA:0,CLK:0, NOT_MCLR:0}      
        if self.uno1.write_digital() == False:
            self.output_text.insert(END,'ERROR - Comms Failure - Program Aborted') 
            self.output_text.see(END)
            return
        self.enter_program_verify()
        self.clock_data_write(self.bulk_erase_program_memory_i)  
        time.sleep(.1)         
        self.clock_data_write(self.reset_address_i)
        for n in range (0,self.pc+1):
            self.load_program_memory(n, self.p_write[n])
            if (n+1)%16 == 0 : #12F1822 has 16 latches for temporary storage
                self.clock_data_write(self.begin_programming_internally_timed_i)
                time.sleep(0.1)

            self.increment_address(1)

        if (n+1)%16 != 0: # check for unprogrammed latches
            self.clock_data_write(self.begin_programming_internally_timed_i)

        self.load_configuration()#0x8000        
        self.increment_address(7)#0x8007
        self.load_program_memory(0x8007, self.p_write[0x8007])  
        self.clock_data_write(self.begin_programming_internally_timed_i)
        time.sleep(.1) 
        self.increment_address(1)#0x8008
        self.load_program_memory(0x8008, self.p_write[0x8008])  
        self.clock_data_write(self.begin_programming_internally_timed_i)
        self.exit_program_verify()    
        self.btn_read.configure(state = NORMAL)
        self.btn_load.configure(state = NORMAL)
        self.output_text.insert(END, '\nLoading complete\n')
        self.output_text.see(END)     

   
    def read(self):
        self.uno1 = uno_slave.Uno(address = 1, project = 4, config_digital = [DATA, CLK, NOT_MCLR])
        self.uno1.write_digital_dict = {DATA:0,CLK:0, NOT_MCLR:0}      
        if self.uno1.write_digital() == False:
            self.output_text.insert(END,'ERROR - Comms Failure - Program Aborted') 
            self.output_text.see(END)
            return
        self.enter_program_verify()
        #self.clock_data_write(self.reset_address_i)
        for n in range (0,self.pc+1):
            self.read_program_memory(n)
            self.increment_address(1)

        self.load_configuration()#0x8000        
        self.increment_address(7) #0x8007
        self.read_program_memory(0x8007)
        self.increment_address(1)        
        self.read_program_memory(0x8008)        
        self.exit_program_verify()    
        self.output_text.insert(END, '\nReading complete\n')
        self.output_text.see(END)
        self.btn_read.configure(state = NORMAL)
        self.btn_load.configure(state = NORMAL)

    def enter_program_verify(self): 
        self.uno1.write_digital_dict[NOT_MCLR] = 1
        self.uno1.write_digital() 
        self.uno1.write_digital_dict[NOT_MCLR] = 0
        self.uno1.write_digital() 

        self.clock_data_write(self.enter_program_verify_i)
        self.uno1.write_digital_dict[CLK] = 1
        self.uno1.write_digital()
        self.uno1.write_digital_dict[CLK] = 0
        self.uno1.write_digital() 
      
    def load_program_memory(self, address, program_word):
        self.output_text.insert(END, 'address = 0x{0:x} loading = 0x{1:x}\n'.format(address, program_word))
        self.output_text.see(END)    
        self.clock_data_write(self.load_program_memory_i)
        program_data =self.make_list((program_word << 1) & 0x7FFE, 16)    #moves 14 bit data to the middle ie xdatax (16 bits)        
        self.clock_data_write(program_data)  
   
    def increment_address(self, num):
      for n in range (0,num):
        self.clock_data_write(self.increment_address_i)

    def load_configuration(self):
        self.clock_data_write([0]*6)
        self.clock_data_write([0]*16) # This function is only called to get to 0x8000, this is dummy data 

    def read_program_memory(self,address):
        self.clock_data_write(self.read_program_memory_i)
        self.uno1 = uno_slave.Uno(address = 1, project = 4, config_digital = [CLK, NOT_MCLR])
        self.uno1.write_digital_dict[CLK] = 1
        self.uno1.write_digital()
        self.uno1.write_digital_dict[CLK] = 0
        bit_lst = []
        for n in range (0,16):
            self.uno1.write_digital_dict[CLK] = 1
            self.uno1.write_digital()
            self.uno1.write_digital_dict[CLK] = 0
            self.uno1.write_digital()                   
            self.uno1.read_digital()            
            bit_lst.append(self.uno1.read_digital_dict[DATA])           

        bit_lst.reverse()
        word = 0            
        for bit in bit_lst:
            word = (word + int(bit)) << 1
                
        word = (word >> 2) & 0x3FFF
        self.uno1 = uno_slave.Uno(address = 1, project = 4, config_digital = [DATA, CLK, NOT_MCLR])
        self.output_text.insert(END, 'address = 0x{0:x} program = 0x{1:x} read = 0x{2:x}\n'.format(address, self.p_write[address], word)) 
        self.output_text.see(END)
             
    def read_data_memory(self,address):
        self.clock_data_write(self.read_data_memory_i)
        self.uno1 = uno_slave.Uno(address = 1, project = 4, config_digital = [CLK, NOT_MCLR])
        self.uno1.write_digital_dict[CLK] = 1
        self.uno1.write_digital()
        self.uno1.write_digital_dict[CLK] = 0
        bit_lst = []
        for n in range (0,15):
            self.uno1.write_digital_dict[CLK] = 1
            self.uno1.write_digital()
            self.uno1.write_digital_dict[CLK] = 0
            self.uno1.write_digital()
            self.uno1.read_digital()
            bit_lst.append(self.uno1.read_digital_dict[DATA])

        bit_lst.reverse()
        word = 0            
        for bit in bit_lst:
            word = (word + int(bit)) << 1
                
        word = (word >> 2) & 0x3FFF
        self.uno1 = uno_slave.Uno(address = 1, project = 4, config_digital = [DATA, CLK, NOT_MCLR])
        self.output_text.insert(END, 'address = 0x{0:x} program = 0x{1:x} read = 0x{2:x}\n'.format(address, self.p_write[address], word)) 
        self.output_text.see(END)        
                   
    def exit_program_verify(self):
        self.uno1.write_digital_dict[NOT_MCLR] = 1
        self.uno1.write_digital()
        
    def make_list(self, value, numbits):
        lst = []
        for n in range (0, numbits):
            lst.append(value & 0x01)
            value = value >> 1

        return lst

    def clock_data_write(self, bit_list):
        for bit in bit_list:
            self.uno1.write_digital_dict[DATA] = bit
            self.uno1.write_digital_dict[CLK] = 1
            self.uno1.write_digital()
            self.uno1.write_digital_dict[CLK] = 0
            self.uno1.write_digital()
    
tk = Tk()
programmer = Pic12f1822(tk)
tk.mainloop()
