#!/usr/bin/python3
#prog_07_06.py (bmpdraw.py)

import struct
from math import  *

class BmpDraw:

    def __init__(self, infile_name):
        self.infile_name = infile_name
        self.read_template()
        
    def read_template(self):
        self.flag = False   
        try:
            template_file = open(self.infile_name, 'rb')
            self.bit_map = template_file.read()
            template_file.close()
        except:
            print('Failed no template file!')
            return 
        
        if self.bit_map[0:2] != 'BM'.encode('utf-8'):
            print('Failed not a .bmp!')
            return 

        self.flag = True
        self.header = self.bit_map[0:54]
        self.byte_s = self.bit_map[54:]
        self.width=(struct.unpack('<I', self.header[18:22]))[0]
        if self.width%4 > 0:
            print('This class can only proces bit maps with a width divisible by 4.')

        self.height=(struct.unpack('<I', self.header[22:26]))[0]
        self.bit_map_dict = {}
        for y in range (0, self.height):
            for x in range (0, self.width):
                self.key=(x, y)
                self.pixel_list = []
                for c in range (0,3):
                    t = self.byte_s[3 * (y * self.width + x) + c]
                    self.pixel_list.append(t)
                    self.bit_map_dict[self.key] = self.pixel_list

        self.flag = True

    def line(self, x1, y1, x2, y2, colour):
        if x1 != x2:
            angle = atan2((y2-y1),(x2-x1))
        elif y2  > y1:
            angle = pi/2
        else:
            angle = 3 * pi/2
        length = int(sqrt((y2 - y1)**2+( x2 - x1) ** 2))
        for n in range (0,length):
            x=int(x1 + n * cos(angle))
            y=int(y1 + n * sin(angle))
            self.bit_map_dict[(x, y)] = colour
	    
    def poly(self, points, colour):
        l = len(points)
        for n in range (0, l ,2):
            x1=points[n]
            y1=points[n + 1]
            if n + 3 < l:
                x2 = points[n + 2]
                y2 = points[n + 3]
            else:
                x2=points[0]
                y2=points[1]

            if x1 != x2:
                angle=atan2((y2-y1),(x2-x1))
            elif y2 > y1:
                angle=pi/2
            else:
                angle = 3 * pi/2
            length = int(sqrt((y2 - y1)**2+(x2 - x1) ** 2))
            for n in range (0, length):
                x=int(x1+n * cos(angle))
                y=int(y1 + n* sin(angle))
                self.bit_map_dict[(x, y)] = colour

    def arc(self, xc , yc, r , colour):
        step_angle = 1.0/r
        number_of_steps = int(r * 2 * pi)
        for n in range (0, number_of_steps):
            x = int(xc+r * cos(n * step_angle))
            y = int(yc+r * sin(n * step_angle))
            self.bit_map_dict[(x, y)]=colour

    def write_file(self,out_file):
            out_byte_a = bytearray(self.header)
            for y in range (0, self.height):
                for x in range (0, self.width):
                    key = (x, y)
                    for c in range (0, 3):
                        colour = self.bit_map_dict[key][c]
                        colour_byte = bytes([colour])   
                        out_byte_a.extend(colour_byte)

            output_file=open(out_file, 'wb')
            output_file.write(out_byte_a)
            output_file.close()

if __name__  == '__main__':
    
    draw=BmpDraw('template_bmp_draw_test.bmp')
    if draw.flag:
        draw.arc(draw.width/2, draw.height/2, draw.height/4, [255, 0, 0])
        draw.line(0,0,draw.width,draw.height,[0, 0, 255])
        draw.poly([10,10,100,10,100,300],[100, 100, 100])
        draw.poly([20, 20, 200, 20, 200, 400, 20, 400], [255, 127, 0])
        draw.write_file('output.bmp')
