#!/usr/bin/env python3
#prog_06_01 (plotter.py)

import tkinter 

class Graph(tkinter.Frame):
    def __init__(self, parent,width, height, scale_x = 1, scale_y = 1):
        tkinter.Frame.__init__(self,parent)
        self.width = width
        self.height = height
        self.scale_x = scale_x
        self.scale_y = scale_y
        self.frm = tkinter.Frame(parent, borderwidth = 2, relief = 'raised')
        self.can = tkinter.Canvas(self.frm,width = self.width, height = self.height, bg = '#CCCCCC')
        self.btn_reset = tkinter.Button(self.frm, text = 'Reset', command = self.reset)   
        self.btn_flyback = tkinter.Button(self.frm, text = 'Flyback', command = self.flyback) 
        self.lbl_measurement = tkinter.Label(self.frm)

        self.can.grid(row = 0, column = 0, rowspan = 3)
        self.btn_reset.grid(row = 0, column = 1, padx = 5)
        self.lbl_measurement.grid(row = 1, column = 1, padx = 5)
        self.btn_flyback.grid(row = 2, column = 1, padx = 5)
        
        self.can.bind("<Button-3>",self.mouse_rh)
        self.can.bind("<Button-1>", self.mouse_lh)
        self.measurements = []
        self.frm.grid()
        self.flyback()
   
    def plot(self, *args):
        palette = ['#FF0000','#00FF00','#0000FF','#FFFF00','#00FFFF','#000000']    
        if self.x < self.width:
            for channel, y in enumerate(args):
                y = args[channel]               
                if self.x == 0:
                    self.y_last[channel] = y

                self.can.create_line(self.x - 1, self.height - self.y_last[channel], self.x, self.height - y,
                                    fill = palette[channel])      
                
                self.y_last[channel] = y
            
            self.x = self.x + 1
    
    def reset(self):
        self.can.delete(tkinter.ALL)
        self.flyback()
    
    def flyback(self):
        self.x = 0
        self.y_last = [0, 0, 0, 0, 0, 0]

    def mouse_lh(self, event):
        x = (event.x) * self.scale_x/self.width
        y = (self.height - event.y) * self.scale_y/self.height
        string = '{0:0.2f},{1:0.2f}'.format(x,y) 
        measurement = self.can.create_text(event.x, event.y, text = string, font = ('Helvetica',8),\
                               fill = '#000000')       
        self.measurements.append(measurement)
               
        self.lbl_measurement.configure(text = string, font = ('Helvetica', 8))

    def mouse_rh(self, event):
        for measurement in self.measurements:
            self.can.delete(measurement)
 
        self.measurements = []  

print(__name__)
'''The following code is for standalone testing'''

def test():
    global x                                           #See comments on global below
    phase = math.radians(30)        
    y0 = 200 * math.sin(x / 30) + 200
    y1 = 200 * math.sin(x / 30  + phase) + 200
    y2 = 200 * math.sin(x / 30  + phase * 2) + 200
    y3 = 200 * math.sin(x / 30  + phase * 3) + 200
    y4 = 200 * math.sin(x / 30  + phase * 4) + 200
    y5 = 200 * math.sin(x / 30  + phase * 5) + 200
    x = x +  1                          #Global x is used to hold the x position after this function ends    
    graph.plot(y0, y1, y2, y3, y4, y5)
    graph.after(10,test)

if __name__ == "__main__":
    import math

    root = tkinter.Tk()
    graph = Graph(root, 800,400)
    root.title('Sine Wave Plotting Test')
    x = 0                                 #Initial assignment of global x
    
    graph.grid()
    test()
    root.mainloop()