#!/usr/bin/env python3
#current_monitor
#prog_08_01.py

import tkinter as tk
import tkinter.filedialog
import tkinter.messagebox 
import pickle
import collections
import uno_slave
import plotter 
import bmp_draw

class CurrentLogger(tk.Frame):
    def __init__(self, master):
        self.master = master
        self.uno1 = uno_slave.Uno(address = 1, project = 4, config_digital = [])
        self.master.title('Current Monitor')
        self.Y = 500
        self.X = 720
        self.x  = 0
        try:
            calibration_file = open('current_monitor_cal.dat', 'rb')
            self.calibration_dict = pickle.load(calibration_file)
            calibration_file.close()
        except:
            self.calibration_dict = {'raw_offset':0, 'raw_to_current':1, 'fullscale_current':100}

        self.circ_buffer = collections.deque([], maxlen=50)
        #add menus 
        menubar = tk.Menu(self.master)
        master.config(menu=menubar)                    
        commissioning_menu = tk.Menu(menubar, tearoff = 0)
        menubar.add_cascade(label = 'Commissioning', menu = commissioning_menu)
        commissioning_menu.add_command(label = 'Calibration', command = self.calibration)
        commissioning_menu.add_command(label = 'Future Use')
        commissioning_menu.add_command(label = 'Future Use')
        commissioning_menu.add_separator()
        commissioning_menu.add_command(label = 'Future Use')
        screen_capture_menu = tk.Menu(menubar, tearoff = 0)         
        menubar.add_cascade(label = 'Screen Capture', menu = screen_capture_menu)        
        screen_capture_menu.add_command(label = 'Capture', command = self.capture)
   
        self.num_samples = self.X    
        self.graph1 = plotter.Graph(master,self.X, self.Y,
         scale_x = self.num_samples, scale_y = self.calibration_dict['fullscale_current'])   
        self.graph1.frm.grid(row = 0, column = 0, padx = 10, pady =10,columnspan = 3)   
        self.reading()
        
    def reading(self):
        self.uno1.read_analog(1)
        raw = self.uno1.read_analog_dict[0]  
        i = (raw-self.calibration_dict['raw_offset']) * self.calibration_dict['raw_to_current']
        y = int(i*self.Y/self.calibration_dict['fullscale_current'])
        self.circ_buffer.append([raw,i,y])          
        if self.x%self.X == 0:
            self.graph1.flyback()       
           
        self.graph1.plot(y)            
        self.x = self.x + 1            
        self.master.after(10000, self.reading)   

    def calibration(self):
        self.cal_window = tk.Toplevel()
        self.cal_window.title('Calibration')
        self.cal_output = tk.Text(self.cal_window, width = 30, height = 20, relief = 'ridge')
        self.btn_save_cal = tk.Button(self.cal_window, text = 'Save Calibration', command = self.save_calibration)
        self.lbl_raw = tk.Label(self.cal_window, text = 'Raw/Current/Screen Position')
        self.lbl_offset = tk.Label(self.cal_window, text = 'raw_offset')
        self.strvar_offset = tk.StringVar()
        self.strvar_offset.set(self.calibration_dict['raw_offset'])
        self.ent_offset = tk.Entry(self.cal_window, textvariable = self.strvar_offset, width = 10)
        self.lbl_raw_to_current = tk.Label(self.cal_window, text = 'raw_to_current')
        self.strvar_raw_to_current = tk.StringVar()
        self.strvar_raw_to_current.set(self.calibration_dict['raw_to_current'])
        self.ent_raw_to_current = tk.Entry(self.cal_window, textvariable = self.strvar_raw_to_current, width = 10)
        self.lbl_fullscale_current = tk.Label(self.cal_window, text = 'fullscale_current')
        self.strvar_fullscale_current = tk.StringVar()
        self.strvar_fullscale_current.set(self.calibration_dict['fullscale_current'])
        self.ent_fullscale_current = tk.Entry(self.cal_window, textvariable = self.strvar_fullscale_current, width = 10)
        self.cal_window.grid()
        self.lbl_raw.grid(row = 0, column = 0)
        self.cal_output.grid(row = 1, column = 0, rowspan  = 7)
        self.lbl_offset.grid(row = 1, column = 1, sticky = 'S')
        self.ent_offset.grid(row = 2, column = 1, sticky = 'N')
        self.lbl_raw_to_current.grid(row = 3, column = 1, sticky = tk.S)
        self.ent_raw_to_current.grid(row = 4, column = 1,  sticky = tk.N)
        self.lbl_fullscale_current.grid(row = 5, column = 1, sticky = tk.S)
        self.ent_fullscale_current.grid(row = 6, column = 1, sticky = tk.N)
        self.btn_save_cal.grid(row = 7, column = 1, padx = 5, pady = 5, sticky = tk.S)                        
        self.calibration_update()

    def calibration_update(self):     
        try:
            self.cal_output.delete(1.0, tk.END)
            for data in self.circ_buffer:
                self.cal_output.insert(tk.END,'raw = {0:d} i = {1:0.2f}A y = {2:d}\n'.format(data[0], data[1], data[2]))
                self.cal_output.see(tk.END)
        except:
            self.master.after_cancel(self.after_handle)        
        
        self.after_handle =  self.master.after(10000, self.calibration_update)

    def save_calibration(self):
        self.calibration_dict['raw_offset'] = float(self.strvar_offset.get())
        self.calibration_dict['raw_to_current'] = float(self.strvar_raw_to_current.get())
        self.calibration_dict['fullscale_current']= float(self.strvar_fullscale_current.get())
        try:
            calibration_file = open('current_monitor_cal.dat','bw')
            pickle.dump(self.calibration_dict, calibration_file)
            calibration_file.close()
        except:
            self.cal_output.insert(END,'\vFailed to open file to save calibration\n')
            self.cal_output.see(END)

    def capture(self):
        file_name = tk.filedialog.asksaveasfilename(filetypes = [('Bit Map','*.bmp')],title = 'Choose or Enter File Name')
        if file_name:
            segments = []
            tags = self.graph1.can.find_all()                       # find all objects on the canvas
            for tag in tags:                                        # loop through objects
                if self.graph1.can.type(tag) == 'line':             # filter out the lines
                    segments.append(self.graph1.can.coords(tag))    # make a list of coordinate lists

            draw1 = bmp_draw.BmpDraw('current_monitor_screen_cap_template.bmpt')
            if draw1.flag == True:
                for x in range (0,self.X, 90):
                    draw1.line(x, 0, x, self.Y, [0,255,0])
                
                for y in range (0, self.Y, 50 ):
                    draw1.line(0, y, self.X, y, [0,255,0])

                for segment in segments:
                    draw1.line(segment[0], self.Y - segment[1], segment[2], self.Y - segment[3], [0,0,255])
            
            draw1.write_file(file_name)
            tk.messagebox.showinfo('Capture', 'Capture Saved')
        else:
            tk.messagebox.showinfo("Capture", 'Capture Cancelled')
     
tkin = tk.Tk()
front_end = CurrentLogger(tkin)
tkin.mainloop()
