#!/usr/bin/python3
import serial
#import struct
import os
import time
#import struct
import subprocess
from tkinter import  *



FUNCODE_GET_IDS = 8 #maps to FUN_CODE_TX_IDs in the arduino code
FUNCODE_SET_IDS = 9


def scan():
    listbox_usb_ports.delete(0,END)
    acms = []
    file_list = os.listdir('/dev')
    for fl in file_list:
        #if fl[3:6] == 'ACM' or fl[3:6] == 'USB':
        if 'ACM' in fl or 'USB' in fl:
            listbox_usb_ports.insert(END,fl)


def bin_to_ascii(telegram_bin):
    telegram_ascii = bytearray()
    bin2ascii = {0:48,1:49,2:50,3:51,4:52,5:53,6:54,7:55,8:56,9:57,10:65,11:66,12:67,13:68,14:69,15:70}    
    for byte in telegram_bin:
            telegram_ascii.append(bin2ascii[byte >> 4] )
            telegram_ascii.append(bin2ascii[byte & 0x0F])
    return telegram_ascii

def ascii_to_bin(telegram_ascii):
    ascii2bin = {48:0,49:1,50:2,51:3,52:4,53:5,54:6,55:7,56:8,57:9,65:10,66:11,67:12,68:13,69:14,70:15}
    telegram_bin = bytearray()
    
    while telegram_ascii:
        a = telegram_ascii.pop(0)
        a = ascii2bin[a] << 4
        b = telegram_ascii.pop(0)
        b = ascii2bin[b] 
        telegram_bin.append(a + b)

    return telegram_bin    


def set_ids(*args):
    lbl_output.configure(text = '')
    try:
     
        project = int(strvar_project.get())
        project_hi = (project >> 8) & 0xff
        project_lo = project & 0xff
        addr = int(strvar_address.get())
        
        index = listbox_usb_ports.curselection()[0] 
        acm = '/dev/' + listbox_usb_ports.get(index)
        print('port selected = ', acm)
        ser = (serial.Serial(acm ,baudrate=9600,timeout=0.1,bytesize = 8, stopbits = 2,dsrdtr= False))
        telegram_bin = bytearray()
        telegram_bin.append(255)#broadcast address
        telegram_bin.append(FUNCODE_SET_IDS)
        telegram_bin.append(addr)
        telegram_bin.append(project_hi)
        telegram_bin.append(project_lo)
        fcs= check_sum(telegram_bin,5)
        telegram_bin.append(fcs)

        telegram_ascii_tx = b':' + bin_to_ascii(telegram_bin) + b'\n'
        
        
        print('telegram_tx = ', telegram_ascii_tx)
        ser.write(telegram_ascii_tx)
    except:
        lbl_output.configure(text = 'Choose a USB port')

    try:
        telegram_string = ser.read(64)
        telegram_received = bytearray(telegram_string)
        print('telegram_received = ', telegram_received)

        chk_sum = check_sum(telegram_received,len(telegram_received)-1)

    except Exception as e:
        pass

    try:
        ser.close()
    except:
        pass

def get_ids():
    lbl_output.configure(text = '')
    
    try:
        index = listbox_usb_ports.curselection()[0]
        acm = '/dev/' + listbox_usb_ports.get(index)
        print('port selected = ', acm)
        ser = (serial.Serial(acm ,9600,timeout=0.1,bytesize = 8, stopbits = 2,dsrdtr= False))


        
        telegram_bin = bytearray()
        telegram_bin.append(255)#broadcast address
        telegram_bin.append(FUNCODE_GET_IDS)
        fcs= check_sum(telegram_bin,2)
        telegram_bin.append(fcs)
        
        telegram_ascii_tx = b':' + bin_to_ascii(telegram_bin) + b'\n'
        ser.write(telegram_ascii_tx)
        print('telegram_tx = ',telegram_ascii_tx)
    except:
       lbl_output.configure(text = 'Check USB port choice')

    try:
        rc_ch = 0
        telegram_string = b''
        bytecount = 0
        while (not rc_ch == b'\n') and bytecount < 16:
            rc_ch = ser.read()
            telegram_string = telegram_string + rc_ch    
            bytecount+=1
        if bytecount > 15:
            return
         
        telegram_received_ascii  = bytearray(telegram_string)
        print("telegram received = ",telegram_received_ascii)
        if not telegram_received_ascii[0] == 58:
            return
        
        telegram_received_ascii.pop(0)
        telegram_received_ascii.pop()
        telegram_received_binary = ascii_to_bin(telegram_received_ascii)
        
                               
        print('telegram received binary',list(telegram_received_binary))
        chk_sum = check_sum(telegram_received_binary,len(telegram_received_binary[:-1]))
        print('check sum calculated = ',chk_sum, 'check sum received = ', telegram_received_binary[-1])
        
        strvar_address.set(telegram_received_binary[0])
        project = (telegram_received_binary[2] << 8) + telegram_received_binary[3] 
        strvar_project.set(str(project))
        
    except:
        lbl_output.configure(text = 'Check USB port choice')
    try:
        ser.close()
    except Exception as e:
        pass
        
def check_sum(frame,l):
    frame = frame[:l]
    chk_sum = 0;
    for n in frame:
        chk_sum = chk_sum + n
        #<<
    
    return chk_sum%256

root = Tk()


root.title('Arduino Slave ID Setting Tool')
btn_scan = Button(root, text = 'Scan', command = scan)
strvar_project = StringVar()
ent_project   = Entry(root, textvariable = strvar_project)
lbl_project   = Label(root, text = 'Project ID Number')
strvar_address = StringVar()
ent_address   = Entry(root, textvariable = strvar_address)
lbl_address   = Label(root, text = 'Slave Address')
listbox_usb_ports = Listbox(root, selectmode ='single')
btn_set = Button(root, text = 'Set', command = set_ids)
btn_get = Button(root, text = 'Get', command = get_ids)

lbl_output = Label(root)


#column 0   ******Geometry******
listbox_usb_ports.grid(column = 0, row = 0,rowspan = 3)
lbl_address.grid(column = 0, row = 3)
ent_address.grid(column = 0, row = 4)
lbl_output.grid(column = 0, row = 5, columnspan = 2,sticky = W)

#column 1
btn_scan.grid(column = 1, row = 0)
btn_get.grid(column = 1, row = 1, sticky = S)
btn_set.grid(column = 1, row = 2, sticky = N)

lbl_project.grid(column = 1, row = 3)
ent_project.grid(column = 1, row = 4)



for child in root.winfo_children():
    child.grid_configure(padx=5, pady=5)
    
root.mainloop()

