#!/usr/bin/env python3
#prog_10_13.py

import curses

class ThumbWheel:  
    def __init__(self, std_scr, start_y = 0, start_x = 0, colorpair = 0):
        curses.mousemask(curses.BUTTON1_PRESSED)
        self.stdscr = std_scr        
        self.start_x = start_x
        self.start_y = start_y  
        self.win = curses.newwin(3, 11, start_y , start_x)
        self.win.bkgd(' ', curses.color_pair(colorpair) | curses.A_BOLD)
        self.win.box()
        self.digit = {2:0, 3:0, 4:0, 5:0, 7:0, 8:0, 9:0} 
        self.sign = 1
        self.value = 0.0
        self.update_wheels()

    def inc_dec(self,x, y):
        if x > 1 and x < 10 and x != 6:
            if y == 0:
                self.digit[x] = self.digit[x] + 1           

            if y == 2:
                self.digit[x] = self.digit[x] - 1

            if self.digit[x] > 9: 
                self.digit[x] = 0

            if self.digit[x] < 0:
                self.digit[x] = 9 

    def poll(self, mouseevent):
        _, x, y, _, event_type = mouseevent
        if event_type == curses.BUTTON1_PRESSED:
            x = x - self.start_x
            y = y - self.start_y 

            if x == 1 and y >= 0 and y <= 2:
               self.sign = -1 * self.sign
            else:
                self.inc_dec(x, y)

            self.update_wheels()

            self.value = self.sign * (1000*self.digit[2] + 100*self.digit[3] + 10*self.digit[4] + self.digit[5] + self.digit[7]/10 + 
                              self.digit[8]/100 + self.digit[9]/1000)

    def write(self, new_value):
        self.value = new_value
        if new_value >= 0:
            s = '+{0:08.3f}'.format(new_value)
            self.sign = 1
        else:
            s = '{0:09.3f}'.format(new_value)
            self.sign = -1
    
        self.digit[2] = int(s[1])
        self.digit[3] = int(s[2])
        self.digit[4] = int(s[3])
        self.digit[5] = int(s[4])
        self.digit[7] = int(s[6])
        self.digit[8] = int(s[7])
        self.digit[9] = int(s[8])
        self.update_wheels()

    def update_wheels(self):
            if self.sign == 1:
                sign_string = '+'
            else:
                sign_string = '-'

            self.win.addstr(1, 1, '{0:s}{1:d}{2:d}{3:d}{4:d}.{5:d}{6:d}{7:d}'.format(sign_string, self.digit[2],self.digit[3],
                                                                                      self.digit[4],self.digit[5],self.digit[7],
                                                                                      self.digit[8],self.digit[9]))

if __name__ == '__main__':   
    def main(stdscr):
        curses.init_pair(1, curses.COLOR_WHITE, curses.COLOR_YELLOW)
        curses.curs_set(0)
        stdscr.timeout(0)
        twheel1 = ThumbWheel(stdscr, start_y = 5, start_x = 10)
        twheel2 = ThumbWheel(stdscr, start_y = 10,  start_x = 10, colorpair = 1)
        twheel1.write(1.23456789)
        twheel2.write(-9.98765432)
        while True:
            event = stdscr.getch()
            if event == curses.KEY_MOUSE:
                mouse_event = curses.getmouse()     
                twheel1.poll(mouse_event)
                twheel2.poll(mouse_event)
            if event == ord('q'):
                break

            stdscr.addstr(6,30, '{0:f}'.format(twheel1.value), curses.A_BOLD)
            stdscr.addstr(11,30, '{0:f}'.format(twheel2.value), curses.A_BOLD)
            stdscr.refresh()
            twheel1.win.refresh()
            twheel2.win.refresh()
            curses.napms(100)


    curses.wrapper(main)
    