#include <EEPROM.h>

#define STATE_IDLE                  0
#define STATE_CHECK_TELEGRAM        1
#define STATE_CONFIGURE_IO          2
#define STATE_DIG_READ              3
#define STATE_DIG_WRITE             4
#define STATE_AN_READ               5
#define STATE_AN_WRITE              6

#define STATE_FAIL_SAFE             7
#define STATE_TX_IDS                8
#define STATE_SET_IDS               9
#define STATE_RECEIVING_BYTES       10


#define FUN_CODE_CONFIG             2
#define FUN_CODE_DIG_READ           3
#define FUN_CODE_DIG_WRITE          4
#define FUN_CODE_AN_READ            5
#define FUN_CODE_AN_WRITE           6
#define FUN_CODE_TX_IDS             8
#define FUN_CODE_SET_IDS            9

#define BROADCAST_ADDR              255

#define ERROR_CHECK_SUM             1
#define ERROR_FUN_CODE              2  
#define ERROR_ADDRESS               3  
#define ERROR_PROJECT_ID            4  

#define SLAVE_ADDRESS_EEPROM_LOCATION  0                                                     
#define PROJECT_ID_H_EEPROM_LOCATION   1                                                                        
#define PROJECT_ID_L_EEPROM_LOCATION   2 
 
unsigned char request_telegram_ascii[64]; 
unsigned char request_telegram_binary[32]; 
unsigned char response_telegram_ascii[64];
unsigned char response_telegram_binary[32];
unsigned char telegram_index = 0;
unsigned char telegram_binary_length; 
unsigned char slave_address;
unsigned char project_id_h;
unsigned char project_id_l;
unsigned char state = 0;
volatile unsigned char chrbuff;
unsigned char pullupmask_d;
unsigned char pullupmask_b;


void setup() {

  Serial.begin(9600,SERIAL_8N2);
  Serial.setTimeout(100);
  slave_address = EEPROM.read(SLAVE_ADDRESS_EEPROM_LOCATION);                                                     
  project_id_h  = EEPROM.read(PROJECT_ID_H_EEPROM_LOCATION);                  
  project_id_l  = EEPROM.read(PROJECT_ID_L_EEPROM_LOCATION); 
  if (slave_address == 255){
      slave_address = 0;
    }

}
  

unsigned char ascii_to_binary(unsigned char * datagram_ascii, unsigned char * datagram_binary, unsigned char datag_len){
unsigned char a,b,c,d,n = 0; 
  for (n = 0; n < datag_len; n+=2){
    a = datagram_ascii[n];
    b = datagram_ascii[n+1]; 
    
    switch(a){
      case '0' : a = 0 ; break;
      case '1' : a = 1 ; break;
      case '2' : a = 2 ; break;
      case '3' : a = 3 ; break;
      case '4' : a = 4 ; break;
      case '5' : a = 5 ; break;
      case '6' : a = 6 ; break;
      case '7' : a = 7 ; break;
      case '8' : a = 8 ; break;
      case '9' : a = 9 ; break;
      case 'A' : a = 10 ; break;
      case 'B' : a = 11 ; break;
      case 'C' : a = 12 ; break;
      case 'D' : a = 13 ; break;
      case 'E' : a = 14 ; break;
      case 'F' : a = 15 ; break;
    }
    switch(b){
        case '0' : b = 0 ; break;
        case '1' : b = 1 ; break;
        case '2' : b = 2 ; break;
        case '3' : b = 3 ; break;
        case '4' : b = 4 ; break;
        case '5' : b = 5 ; break;
        case '6' : b = 6 ; break;
        case '7' : b = 7 ; break;
        case '8' : b = 8 ; break;
        case '9' : b = 9 ; break;
        case 'A' : b = 10 ; break;
        case 'B' : b = 11 ; break;
        case 'C' : b = 12 ; break;
        case 'D' : b = 13 ; break;
        case 'E' : b = 14 ; break;
        case 'F' : b = 15 ; break;
      }
    datagram_binary[n/2] = ((a << 4) & 0xF0) + (b & 0x0F);
    
   }
  return n/2;
   
}
void binary_to_ascii(unsigned char * datagram_binary, unsigned char * datagram_ascii, unsigned char datag_len){
  unsigned char n,a,b;
  for (n = 0; n < datag_len*2 ; n+=2) {
   a = ((response_telegram_binary[n/2] & 0xF0) >> 4) ;
   b = response_telegram_binary[n/2] & 0x0F;
   switch(a){
    case 0 : a = '0'; break;
    case 1 : a = '1'; break;
    case 2 : a = '2'; break;
    case 3 : a = '3'; break;
    case 4 : a = '4'; break;
    case 5 : a = '5'; break;
    case 6 : a = '6'; break;
    case 7 : a = '7'; break;
    case 8 : a = '8'; break;
    case 9 : a = '9'; break;
    case 10 : a = 'A'; break;
    case 11 : a = 'B'; break;
    case 12 : a = 'C'; break;
    case 13 : a = 'D'; break;
    case 14 : a = 'E'; break;
    case 15 : a = 'F'; break;
   }
   switch(b){
    case 0 : b = '0'; break;
    case 1 : b = '1'; break;
    case 2 : b = '2'; break;
    case 3 : b = '3'; break;
    case 4 : b = '4'; break;
    case 5 : b = '5'; break;
    case 6 : b = '6'; break;
    case 7 : b = '7'; break;
    case 8 : b = '8'; break;
    case 9 : b = '9'; break;
    case 10 : b = 'A'; break;
    case 11 : b = 'B'; break;
    case 12 : b = 'C'; break;
    case 13 : b = 'D'; break;
    case 14 : b = 'E'; break;
    case 15 : b = 'F'; break;
     }   
   response_telegram_ascii[n] = a;
   response_telegram_ascii[n+1] = b; 
  }
  
    
}


unsigned char check_sum(volatile unsigned char * frame, unsigned char l){  //CHECK SUM*****************
  int sum=0;
  unsigned char n; 
  for (n = 0 ; n < l; n++){
    sum = sum + frame[n];
  }
  return (unsigned char)(sum%256); 
}


unsigned char set_ids(){ //SET IDS*****************
  if (request_telegram_binary[2] != 255){
  EEPROM.write(SLAVE_ADDRESS_EEPROM_LOCATION,request_telegram_binary[2]);
  EEPROM.write(PROJECT_ID_H_EEPROM_LOCATION,request_telegram_binary[3]);
  EEPROM.write(PROJECT_ID_L_EEPROM_LOCATION,request_telegram_binary[4]);
  }
  slave_address = EEPROM.read(SLAVE_ADDRESS_EEPROM_LOCATION);                                                     
  project_id_h  = EEPROM.read(PROJECT_ID_H_EEPROM_LOCATION);                  
  project_id_l  = EEPROM.read(PROJECT_ID_L_EEPROM_LOCATION); 
  return STATE_TX_IDS;
}

  


  
unsigned char tx_ids(){                                         
  response_telegram_binary[0] = slave_address;
  response_telegram_binary[1] = FUN_CODE_TX_IDS;
  response_telegram_binary[2] = project_id_h;
  response_telegram_binary[3] = project_id_l;
  response_telegram_binary[4] = check_sum(response_telegram_binary,4);
  binary_to_ascii(response_telegram_binary, response_telegram_ascii, 5);
  Serial.write(':');
  Serial.write(response_telegram_ascii,10);
  Serial.write('\n');
  return STATE_IDLE;
}

unsigned char analog_write(){               
  unsigned char n;
  for (n = 2; n < telegram_binary_length;n+=2){     
      analogWrite(request_telegram_binary[n], request_telegram_binary[n+1]);
  }
 
  response_telegram_binary[0] = slave_address;
  response_telegram_binary[1] = FUN_CODE_AN_WRITE;
  response_telegram_binary[2] = check_sum(response_telegram_binary,2);
  binary_to_ascii(response_telegram_binary, response_telegram_ascii, 3);
  Serial.write(':');
  Serial.write(response_telegram_ascii,6);
  Serial.write('\n');
  return STATE_IDLE;
}




unsigned char analog_read(){ 
  unsigned char n, num, m;      
  int val;
  num = request_telegram_binary[2];
  response_telegram_binary[0] = slave_address;
  response_telegram_binary[1] = FUN_CODE_AN_READ;
  m = 2;
  for (n = 0; n < num; n++){
   val = analogRead(n);
   response_telegram_binary[m]   = val >> 8;
   m++;
   response_telegram_binary[m] = val & 0xFF;;
   m++;
  
  }
  response_telegram_binary[m] = check_sum(response_telegram_binary,m);
  m++;
  binary_to_ascii(response_telegram_binary, response_telegram_ascii, m);
  Serial.write(':');
  Serial.write(response_telegram_ascii,2*m);
  Serial.write('\n');





  return STATE_IDLE;
}



unsigned char digital_read(){          
  response_telegram_binary[0] = slave_address;
  response_telegram_binary[1] = FUN_CODE_DIG_READ;
  response_telegram_binary[2] = PIND;
  response_telegram_binary[3] = PINB;
  response_telegram_binary[4] = check_sum(response_telegram_binary,4);
  binary_to_ascii(response_telegram_binary, response_telegram_ascii, 5);
  Serial.write(':');
  Serial.write(response_telegram_ascii,10);
  Serial.write('\n');
  return STATE_IDLE;  
}



unsigned char digital_write(){                                                                                 
  PORTD =request_telegram_binary[2] | pullupmask_d;
  PORTB =request_telegram_binary[3] | pullupmask_b;
  response_telegram_binary[0] = slave_address;
  response_telegram_binary[1] = FUN_CODE_DIG_WRITE;
  response_telegram_binary[2] = check_sum(response_telegram_binary,2);
  binary_to_ascii(response_telegram_binary, response_telegram_ascii, 3);
  Serial.write(':');
  Serial.write(response_telegram_ascii,6);
  Serial.write('\n'); 

  
  return STATE_IDLE;

}



unsigned char configure_io(){           
  DDRD = request_telegram_binary[4] & 0xFC; //to protect the serial pins TX/RX from being set
  DDRB = request_telegram_binary[5]; 
  pullupmask_d = ~request_telegram_binary[4] & 0xFC; 
  pullupmask_b = ~request_telegram_binary[5];          
  PORTD = pullupmask_d;  //inputs set to pull up
  PORTB = pullupmask_b;  //''''-------------''''
  response_telegram_binary[0] = slave_address;
  response_telegram_binary[1] = FUN_CODE_CONFIG;
  response_telegram_binary[2] = project_id_h;
  response_telegram_binary[3] = project_id_l;
  response_telegram_binary[4] = check_sum(response_telegram_binary,4);
  binary_to_ascii(response_telegram_binary, response_telegram_ascii, 5);
  Serial.write(':');
  Serial.write(response_telegram_ascii,10);
  Serial.write('\n'); 
  return STATE_IDLE;
}



unsigned char check_telegram(){  
  unsigned char result,n;
  telegram_binary_length = ascii_to_binary(request_telegram_ascii,request_telegram_binary, telegram_index);
 
  if ( request_telegram_binary[telegram_binary_length-1] != check_sum(request_telegram_binary,telegram_binary_length - 1)){
    error_tx(ERROR_CHECK_SUM);
    return STATE_IDLE;
  }
  telegram_binary_length = telegram_binary_length -1;// Don't need the check sum byte any more
  if (request_telegram_binary[0] == slave_address){
    switch (request_telegram_binary[1]){
      case FUN_CODE_DIG_READ  : result = STATE_DIG_READ;  break;
      case FUN_CODE_DIG_WRITE : result = STATE_DIG_WRITE; break;
      case FUN_CODE_AN_READ   : result = STATE_AN_READ;   break;
      case FUN_CODE_AN_WRITE  : result = STATE_AN_WRITE;  break;
      case FUN_CODE_CONFIG    :
                              if (request_telegram_binary[2] == project_id_h &&
                              request_telegram_binary[3] == project_id_l) result = STATE_CONFIGURE_IO;
                              else{
                                 error_tx(ERROR_PROJECT_ID);
                                 result = STATE_IDLE; 
                                 } ;   break;
      default : error_tx(ERROR_FUN_CODE); 
    }
  }
  else if (request_telegram_binary[0] == BROADCAST_ADDR){  
    switch (request_telegram_binary[1]){
      case FUN_CODE_TX_IDS    : result  = STATE_TX_IDS;  break;
      case FUN_CODE_SET_IDS   : result  = STATE_SET_IDS; break; 
      default : error_tx(ERROR_FUN_CODE);
                result = STATE_IDLE;
    }
}
else {  //error response
    error_tx(ERROR_ADDRESS);
    result = STATE_IDLE;
}
return result;    
} 

void error_tx(unsigned char e){
    response_telegram_binary[0] = slave_address;
    response_telegram_binary[1] = 0x80;
    response_telegram_binary[2] = e;
    response_telegram_binary[3] = check_sum(response_telegram_binary,3);
    binary_to_ascii(response_telegram_binary, response_telegram_ascii, 4);
    Serial.write(':');
    Serial.write(response_telegram_ascii,8);
    Serial.write('\n'); 
  
}



void serialEvent(){
  chrbuff = Serial.read();
  if (chrbuff == ':'){
    state = STATE_RECEIVING_BYTES;
    telegram_index = 0;
  }
  else if (chrbuff == '\n'){
    state = STATE_CHECK_TELEGRAM; /*'\n' is not added to the telegram and 'telegram_index' is now the length
                                  of the telegram*/
  }
  else {
    request_telegram_ascii[telegram_index] = chrbuff;
    telegram_index++; /*telegram_index points to the next location to 
                      be filled and is one more than the telegram length.*/
  }

}


void loop() {
  switch(state){
    case STATE_IDLE                 :telegram_index = 0;         break;
    case STATE_CONFIGURE_IO         :state = configure_io();     break;
    case STATE_DIG_READ             :state = digital_read();     break;
    case STATE_DIG_WRITE            :state = digital_write();    break;
    case STATE_AN_READ              :state = analog_read();      break;
    case STATE_AN_WRITE             :state = analog_write();     break;    
    case STATE_CHECK_TELEGRAM       :state = check_telegram();   break;
    case STATE_TX_IDS               :state = tx_ids();           break;
    case STATE_SET_IDS              :state = set_ids() ;         break;
    case STATE_RECEIVING_BYTES      : if(telegram_index > 32){
                                         state = STATE_IDLE;
                                        } 
                                        break;
    default                         : state = STATE_IDLE;                                 
                                                                            
  }   
}




/*
TELEGRAM FORMATS
=================

BROADCAST GET IDs
--------------------

REQUEST
0 address 255
1 fun code 6
2 FCS 

REPLY
0 slave address
1 fun code 6
2 project id h
3 project id l
4 FCS  

BROADCAST SET IDs
----------------
REQUEST
0 broadcast address 255
1 FUN_CODE_SET_IDS
2 address to set
2 project id hi to set
3 project id lo to set
4 FCS

RESPONSE
0 slave address newly set
1 FUN_CODE_GET_IDS (responds after checking EEPROM with an IDS tx)
2 project id h newly set
3 project id l newly set
4 FCS 

CONFIGURE IO
------------

REQUEST
0 address 
1 fun code 2 (FUN_CODE_CONFIG)
2 project id h
3 project id l 
4 pins 0 - 7 write values
5 pins 8 -13 write values
n FCS

Notes:
Digital pins 0 - 13 
Analogue pins A0 = 16
              A1 = 17
              " " " "
Response
0 address 
1 fun code 2 (FUN_CODE_CONFIG)
2 project id h
3 project id l 
4 FCS

DIGITAL READ
-----------

REQUEST 
0 Address.
1 Funcode 3 (FUN_CODE_DIG_READ).
2 FCS


RESPONSE
0 Address
1 PIND
2 PINB
3 FCS

DIGITAL WRITE
-------------
REQUEST
0 Address
1 FUN_CODE_DIG_WRITE
2 PORTD
3 PORTB
4 FCS


RESPONSE
0 Address
1 FUN_CODE_DIG_WRITE
2 FRC


ANALOG READ
----------

REQUEST
0 Address
1 FUN_CODE_AN_READ
2 Number of pins to read - To Keep it simple reads are contiguous from pin 0 
3 FCS

RESPONSE
0 Address
1 FUN_CODE_AN_READ
2 First channel h
3 First channel l
4 Next channel h
5 Next channel l
.
.
.
n FRC

ANALOG_WRITE
------------
REQUEST
0 address
1 FUN_CODE_AN_WRITE
2 First pin to write to
3 value to write
4 Next pin ro write to
5 Next value
.
.
n FCS

RESPONSE
0 Address
1 FUN_CODE_AN_WRITE
2 FRC
   




 */




